#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "idafile.h"
#include "unixstuff.h"

/*{{{  bool  file_exists (filename)*/
bool  file_exists (filename)
/*------------------------*/
	char  *filename;
/*--------------------------------------------------------------*/
/*	Returns 1 if 'filename' exists and is accessible (see	*/
/* below for the interpretation of this) and 0 otherwise.	*/
/*								*/
/* N.B. In the Unix implementation, this function returns 1	*/
/* only if the file exists *and* the directories leading to it	*/
/* can be searched (i.e. have 'x' permission).			*/
/*	Therefore, if any directory leading to the file *cannot**/
/* be searched the function returns 0, even if the file exists.	*/
/* However, in that case the given 'filename' can't be opened,	*/
/* and therefore cannot be overwritten.				*/
/*	In any other implementation of this function, the	*/
/* important property that must be preserved is that, if the	*/
/* function returns 0, it must be guaranteed that an attempt to	*/
/* open 'filename' will not *delete* or *over-write* a		*/
/* pre-existing file with name 'filename'!			*/
/*								*/
/* N.B. System-dependent features:				*/
/* -------------------------------				*/
/* -- Uses the Unix system call 'access()'.			*/
/*--------------------------------------------------------------*/
{
	return  (access (filename, 0) == 0);
}
/*}}}*/

#ifdef V_IDA

/*{{{  long  modify_time (filename)*/
long  modify_time (filename)
/*------------------------*/
	char  *filename;
/*--------------------------------------------------------------*/
/*	Returns a system-dependent representation of the time	*/
/* that file 'filename' was last modified.			*/
/*								*/
/*	In the Unix case this value is given by the 'st_mtime'	*/
/* field of the structure returned by the system call 'stat()'.	*/
/* Its type is 'time_t', which is assumed to be an integer type	*/
/* -- probably 'long'.  Whatever integer type it is, it is	*/
/* converted to 'long' to isolate the system-dependence.	*/
/*								*/
/*	On other systems, if the representation is not an	*/
/* integer type, the return type of this function, and the	*/
/* format in which this information is written to the 'ida'	*/
/* file (in function 'main'), may have to be modified.		*/
/*								*/
/* System-dependent features:					*/
/* --------------------------					*/
/* -- Uses the Unix system call 'stat()'.			*/
/* -- The representation of the return value (see above).	*/
/*--------------------------------------------------------------*/
{
	struct stat  stat_buf;

	if (stat (filename, &stat_buf) < 0) {	/* shouldn't happen! */
		fprintf (stderr, "Cannot find file \"%s\"\n", filename);
		exit (1);
	}

	return  (long) stat_buf.st_mtime;
}
/*}}}*/
/*{{{  bool  ida_file_up_to_date (ida_first_line, src_filename, src_modify_time)*/
bool  ida_file_up_to_date (ida_first_line, src_filename, src_modify_time)
/*---------------------------------------------------------------------*/
	char  *ida_first_line,  *src_filename;
	long   src_modify_time;
/*--------------------------------------------------------------*/
/*	This function determines whether the existing 'ida'	*/
/* file (whose name is 'ida_filename') relates to the current	*/
/* version of source file 'src_filename', i.e. whether the	*/
/* source filename and last modify time specified in		*/
/* 'ida_first_line' (the first line of the 'ida' file) are	*/
/* exactly 'src_filename' and 'src_modify_time'.  If so it	*/
/* returns 1, otherwise it returns 0.				*/
/*								*/
/* N.B.:							*/
/* (1) Expressed as a 'printf' specification, the exact format	*/
/* of 'ida_first_line' is (or should be!):			*/
/*								*/
/*	("%s%s%s %d \n",  IDA_PREFIX,  IDA_VERSION,  filename,	*/
/*	                  modify_time)				*/
/*								*/
/* where:							*/
/*  IDA_PREFIX  ~  "<<ida>>: "	} see 'unixstuff.h' for		*/
/*  IDA_VERSION ~  "v0.3 "	} exact definition.		*/
/*  'modify_time' is a system-dependent representation of	*/
/* the time the file was last modified.  (In the Unix case this	*/
/* value is an integer type -- 'type_t', which is converted to	*/
/* 'long' to maximise portability; see the description in	*/
/* function 'modify_time ()'.)					*/
/*								*/
/* (2) The caller has already checked that 'ida_first_line'	*/
/* begins with IDA_PREFIX, as it should!			*/
/*								*/
/* (3) System-dependent features:				*/
/* ------------------------------				*/
/* -- The type returned by function 'modify_time ()', and hence	*/
/* the format in which it's written in the 'ida_first_line'!	*/
/*--------------------------------------------------------------*/
{
	char  *p1,  *p2;
	long   ida_modify_time;

	/*{{{  return 0 if 'ida' version number incorrect*/
	/*----------------------------------------------*/ 
	/* Return 0 if 'IDA_VERSION' is incorrect.	*/
	/*----------------------------------------------*/ 
	p1 = ida_first_line + IDA_PREFIX_LEN;	/* 'p1' points to IDA_VERSION,     */
						/* assuming line starts IDA_PREFIX */
	
	if (! *p1) return 0;			/* wrong format -- no IDA_VERSION! */
	
	if (strncmp (p1, IDA_VERSION, IDA_VERSION_LEN))
		return 0;			/* version number incorrect */
	
	
	/*}}}*/
	
	/*{{{  return 0 if (filename != src_filename)*/
	/*--------------------------------------------------------------*/ 
	/* Return 0 if (filename != src_filename), or if format is	*/
	/* incorrect.							*/
	/*--------------------------------------------------------------*/ 
	p1 += IDA_VERSION_LEN;		/* 'p1' points to 'filename'	*/
	
	for (p2 = p1;  *p2 && *p2 != ' ';  p2++)
		;			/* skip to end of 'filename' */
	
	if (! *p2) return 0;		/* wrong format -- no " modify_time"! */
	
	*p2 = '\0';			/* terminate 'filename' with '\0' */
	
	if (strcmp (p1, src_filename))
		return 0;		/* (filename != src_filename) */
	/*}}}*/
	
	/*{{{  return 0 if 'modify_time' incorrect*/
	/*--------------------------------------------------------------*/ 
	/* Return 0 if 'modify_time' is incorrect, or format is wrong.	*/
	/*--------------------------------------------------------------*/ 
	
	/*--------------------------------------------------------------*/
	/* (i) Check that the rest of 'ida_first_line' has exactly the	*/
	/* right format (viz. "modify_time \n"), and assign the		*/
	/* 'modify_time' value to 'ida_modify_time'.			*/
	/*--------------------------------------------------------------*/
	p2++;			/* 'p2' points to start of 'modify_time' */
	
	for (ida_modify_time = 0;  isdigit (*p2);  p2++)
		ida_modify_time = 10 * ida_modify_time + (*p2 - '0');
	
	if (strcmp (p2, " \n"))
		return  0;	/* line has wrong format after 'modify_time' */
	
	/*--------------------------------------*/
	/* (ii) Compare the 'modify_time's.	*/
	/*--------------------------------------*/
	if (src_modify_time != ida_modify_time)
		return  0;	/* 'src_filename' has different 'modify_time' */
	/*}}}*/

	return  1;		/* AOK! */
}
/*}}}*/

#endif
